#include <stdio.h>
#include <stdint.h>

#include <arch/arch.h>

#include "memtest.h"

/* Leave space at the beginning and end of memory for this program and for the
 * stack */
#define SAFE_AREA     0x100000
#define STACK_SIZE    65536
#define BASE_ADDRESS  (volatile datum *) (0x8c000000 + SAFE_AREA)
#define NUM_BYTES_32  (32 * 1024 * 1024 - SAFE_AREA - STACK_SIZE)
#define NUM_BYTES_16  (16 * 1024 * 1024 - SAFE_AREA - STACK_SIZE)

int
main(int argc, char **argv)
{
	uint32_t data, *address;
	unsigned long num_bytes;

	if (mm_top() == 0x8e000000) {
		num_bytes = NUM_BYTES_32;
	}
	else {
		num_bytes = NUM_BYTES_16;
	}

	printf("base address: %p\n", BASE_ADDRESS);
	printf("number of bytes: %lu\n", num_bytes);

	printf("memTestDataBus: ");
	fflush(stdout);
	data = memTestDataBus(BASE_ADDRESS);
	if (data != 0) {
		printf("FAIL: %08lx\n", data);
	}
	else {
		printf("PASS\n");
	}
	fflush(stdout);

	printf("memTestAddressBus: ");
	fflush(stdout);
	address = memTestAddressBus(BASE_ADDRESS, num_bytes);
	if (address != NULL) {
		printf("FAIL (%p)\n", address);
	}
	else {
		printf("PASS\n");
	}
	fflush(stdout);

	printf("memTestDevice: ");
	fflush(stdout);
	address = memTestDevice(BASE_ADDRESS, num_bytes);
	if (address != NULL) {
		printf("FAIL (%p)\n", address);
	}
	else {
		printf("PASS\n");
	}
	fflush(stdout);

	return 0;
}
